import {
    ChangeDetectionStrategy,
    ChangeDetectorRef,
    Component,
    EventEmitter,
    inject,
    Input,
    OnDestroy,
    OnInit,
    Output
} from '@angular/core';
import {
    ButtonCloseDirective,
    ColComponent,
    FormSelectDirective,
    ModalBodyComponent,
    ModalComponent,
    ModalFooterComponent,
    ModalHeaderComponent,
    ModalService,
    ModalTitleDirective,
    ProgressComponent,
    RowComponent
} from '@coreui/angular';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';

import { FormsModule } from '@angular/forms';
import { Subscription } from 'rxjs';
import { TranslocoDirective, TranslocoService } from '@jsverse/transloco';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { ExternalCommandsService, HostRescheduleItem, } from '../../../services/external-commands.service';

import { NgSelectModule } from '@ng-select/ng-select';
import { RequiredIconComponent } from '../../required-icon/required-icon.component';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { HttpErrorResponse } from '@angular/common/http';


@Component({
    selector: 'oitc-service-reset-checktime-modal',
    imports: [
        ButtonCloseDirective,
        ColComponent,
        FaIconComponent,
        ModalBodyComponent,
        ModalComponent,
        ModalFooterComponent,
        ModalHeaderComponent,
        ModalTitleDirective,
        RowComponent,
        TranslocoDirective,
        FormsModule,
        NgSelectModule,
        RequiredIconComponent,
        FormSelectDirective,
        XsButtonDirective,
        ProgressComponent
    ],
    templateUrl: './service-reset-checktime-modal.component.html',
    styleUrl: './service-reset-checktime-modal.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class ServiceResetChecktimeModalComponent implements OnInit, OnDestroy {

    /* This component is used to reset the check time of an HOST - the naming is bad */

    @Input({required: true}) public items: HostRescheduleItem[] = [];
    @Input({required: false}) public helpMessage: string = '';
    @Output() completed: EventEmitter<boolean> = new EventEmitter<boolean>();
    public hasErrors: boolean = false;

    public percentage: number = 0;
    public isSending: boolean = false;

    public state?: any

    private readonly modalService: ModalService = inject(ModalService);
    private readonly notyService: NotyService = inject(NotyService);
    private readonly ExternalCommandsService: ExternalCommandsService = inject(ExternalCommandsService);
    private readonly subscriptions: Subscription = new Subscription();
    private readonly TranslocoService = inject(TranslocoService);

    protected resetType: 'hostOnly' | 'hostAndServices' = 'hostAndServices';

    private cdr = inject(ChangeDetectorRef);

    public hideModal() {
        this.isSending = false;
        this.hasErrors = false;

        this.cdr.markForCheck();

        this.modalService.toggle({
            show: false,
            id: 'serviceResetChecktimeModal'
        });
    }

    public ngOnInit() {
        this.subscriptions.add(this.modalService.modalState$.subscribe((state) => {
            if (state.id === 'serviceResetChecktimeModal' && state.show === true) {
            }
        }));
    }

    public ngOnDestroy() {
        this.subscriptions.unsubscribe();
    }

    public sendCommands(): void {
        this.isSending = true;
        this.items.forEach((item) => {
            item.type = this.resetType;
        });


        this.percentage = 0;
        this.isSending = true;
        this.cdr.markForCheck();
        let count = this.items.length;
        let responseCount: number = 0
        let issueCount: number = 0;

        for (let i in this.items) {
            const item = this.items[i];

            this.ExternalCommandsService.reset_check_time(item).subscribe({
                next: (value: any) => {
                    responseCount++
                    this.percentage = Math.round((responseCount / count) * 100);
                    this.cdr.markForCheck();

                    if (responseCount === count && issueCount === 0) {
                        // The timeout is not necessary. It is just to show the progress bar at 100% for a short time.
                        setTimeout(() => {
                            // All records have been sent successfully. Reset the modal
                            this.isSending = false;
                            this.percentage = 0;
                            this.hasErrors = false;

                            this.hideModal();
                        }, 250);


                        const title = this.TranslocoService.translate('Reset check time');
                        const msg = this.TranslocoService.translate('Command sent successfully. Refresh in 5 seconds');
                        this.notyService.genericSuccess(msg, title);
                        setTimeout(() => {
                            this.completed.emit(true);
                        }, 5000);

                    }

                },
                error: (error: HttpErrorResponse) => {
                    this.cdr.markForCheck();
                    issueCount++
                    this.hasErrors = true;

                    responseCount++;
                    this.percentage = Math.round((responseCount / count) * 100);

                    if (responseCount === count && issueCount > 0) {
                        // The timeout is not necessary. It is just to show the progress bar at 100% for a short time.
                        setTimeout(() => {
                            this.isSending = false;
                            this.percentage = 0;
                        }, 250);

                        this.notyService.genericError();

                        this.completed.emit(false);
                    }
                }
            });
        }
    }
}
