import {
    HostForMapItem,
    HoststatusForMapItem,
    MapitemBase,
    ServiceForMapItem,
    ServicestatusForMapItem
} from '../../components/map-item-base/map-item-base.interface';
import { Map } from '../maps/maps.interface';

export interface MapRoot {
    id: number
    name: string
    title: string
    background?: string | null
    background_x?: number | null
    background_y?: number | null
    background_size_x?: number | null
    background_size_y?: number | null
    refresh_interval?: number
    created?: string
    modified?: string
    mapitems: Mapitem[]
    maplines: Mapline[]
    mapgadgets: Mapgadget[]
    mapicons: Mapicon[]
    maptexts: Maptext[]
    mapsummaryitems: Mapsummaryitem[]

    [key: string]: any; // Add index signature
}

/********************************
 * Mapitems
 ********************************/

export interface Mapitem extends MapitemBase {
    limit?: number
    iconset: string
    type: string
    object_id: number
    show_label: boolean
    label_possition: number
}

export interface Mapline extends MapitemBase {
    limit: any
    iconset: any
    type: string
    object_id: number
    show_label: boolean
}

export interface Mapgadget extends MapitemBase {
    size_x: number
    size_y: number
    limit: any
    gadget: string
    type: string
    object_id: number
    transparent_background: boolean
    show_label: boolean | number
    font_size: number
    metric: any
    output_type: any
    label_possition: any
}


export interface Mapicon extends MapitemBase {
    icon: string
}

export interface Maptext extends MapitemBase {
    text: string
    font_size: number
}

export interface Mapsummaryitem extends MapitemBase {
    size_x: number
    size_y: number
    limit: number
    type: string
    object_id: number
    show_label: boolean
    label_possition: number
}

/********************************
 * Editor
 ********************************/

export interface Helplines {
    enabled: boolean
    size: number
}

/********************************
 * Services
 ********************************/

export interface MapItemMultiItem {
    objectId: number;
    mapId: number;
    type: string;
    uuid: string;
}

export interface MapItemMultiPost {
    items: MapItemMultiItem[];
    disableGlobalLoader: boolean;
}


/********************************
 * Maps
 ********************************/

export interface MapsByStringParams {
    'angular': boolean,
    'filter[Maps.name]': string,
    'selected[]': number[],
    'excluded[]'?: number[]
}

/********************************
 * Mapeditors View
 ********************************/

export interface MapDetailsRoot {
    map: Map
    _csrfToken: string
}

/********************************
 * Map View Component
 ********************************/

export interface MapeditorsViewRoot {
    map: MapRoot
    ACL: Acl
    _csrfToken: string
}

export interface Acl {
    hosts: HostsForAcl
    services: ServicesForAcl
    hostgroups: HostgroupsForAcl
    servicegroups: ServicegroupsForAcl
}

export interface HostsForAcl {
    browser: boolean
    index: boolean
}

export interface ServicesForAcl {
    browser: boolean
    index: boolean
}

export interface HostgroupsForAcl {
    extended: boolean
}

export interface ServicegroupsForAcl {
    extended: boolean
}

/********************************
 * Map Summary Toaster Component
 ********************************/

export interface MapSummaryRoot {
    summary: Summary
    _csrfToken: string
}

export interface Summary {
    Map: Map
    Host: HostForMapItem
    Hosts: HostForSummaryRoot[]
    Hoststatus: HoststatusForMapItem
    Hostgroup: HostgroupForSummary
    HostSummary: HostSummary
    HostIdsGroupByState: number[][]
    NotOkHosts: any[]
    Service: ServiceForMapItem
    Services: ServicesForSummaryRoot[]
    Servicestatus: ServicestatusForMapItem
    Servicegroup: ServicegroupForSummary
    ServiceSummary: ServiceSummary
    ServiceIdsGroupByState: string[][]
    NotOkServices: any[]
    CumulatedHumanState: string
}

export interface ServicesForSummaryRoot {
    Service: ServiceForSummary
    Servicestatus: ServicestatusForMapItem
    Host: {
        display_name: string
    }
}

export interface HostForSummaryRoot {
    Host: HostForSummary
    Hoststatus: HoststatusForMapItem
    ServiceSummary: ServiceSummary
    ServiceIdsGroupByState: string[][]
}

export interface HostForSummary {
    id: number
    hostgroup_id: number
    host_uuid: string
    HostMapping: {
        display_name: string
    }
    name: string
}

export interface ServiceForSummary {
    id: number
    servicegroup_id: number
    uuid?: string
    display_name?: string
    service_uuid: string
    ServiceMapping: {
        display_name: string
    }
    Servicestatus: ServicestatusForSummary
    HostMapping: {
        display_name: string
    }
}

export interface HostSummary {
    state: number[]
    total: number
}

export interface ServiceSummary {
    state: number[]
    total: number
}

export interface ServicegroupForSummary {
    uuid: string
    name: string
    description: string
}

export interface HostgroupForSummary {
    uuid: string
    name: string
    description: string
    HostSummary: HostSummary
    TotalServiceSummary: TotalServiceSummary
}

export interface TotalServiceSummary {
    state: number[]
    total: number
}

export interface ServicestatusForSummary {
    hostname: string
    service_description: string
}

/********************************
 * Maps Service Browser Redirect
 ********************************/

export interface MapsServiceBrowserRedirectRoot {
    service: {
        hostname: string
        service_description: string
    }
}

export interface MapsServiceBrowserRedirectParams {
    angular: boolean
    uuid: string | null
}
