import { inject, Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { PROXY_PATH } from '../../../../tokens/proxy-path.token';
import { catchError, map, Observable, of } from 'rxjs';
import {
    MapDetailsRoot,
    MapeditorsViewRoot,
    MapItemMultiPost,
    MapsServiceBrowserRedirectParams,
    MapsServiceBrowserRedirectRoot
} from './mapeditors.interface';
import { GenericIdResponse, GenericResponseWrapper, GenericValidationError } from '../../../../generic-responses';

@Injectable({
    providedIn: 'root'
})
export class MapeditorsService {
    private readonly http: HttpClient = inject(HttpClient);
    private readonly proxyPath: string = inject(PROXY_PATH);

    public saveMapitemMulti(postData: MapItemMultiPost): Observable<GenericResponseWrapper> {
        const proxyPath = this.proxyPath;
        return this.http.post<any>(`${proxyPath}/map_module/mapeditors/mapitemMulti.json?angular=true`, postData)
            .pipe(
                map(data => {
                    // Return true on 200 Ok
                    return {
                        success: true,
                        data: data as GenericIdResponse
                    };
                }),
                catchError((error: any) => {
                    const err = error.error.error as GenericValidationError;
                    return of({
                        success: false,
                        data: err
                    });
                })
            );
    }

    public loadMapDetails(id: number): Observable<MapDetailsRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<any>(`${proxyPath}/map_module/mapeditors/mapDetails/${id}.json?angular=true`, {}).pipe(
            map(data => {
                return data;
            })
        )
    }

    public loadMapView(id: number): Observable<MapeditorsViewRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<any>(`${proxyPath}/map_module/mapeditors/view/${id}.json?angular=true`, {}).pipe(
            map(data => {
                return data;
            })
        )
    }

    public getMapsServiceBrowserRedirect(params: MapsServiceBrowserRedirectParams): Observable<MapsServiceBrowserRedirectRoot | {
        errors: boolean
    }> {
        const proxyPath = this.proxyPath;
        return this.http.get<MapsServiceBrowserRedirectRoot | {
            errors: boolean
        }>(`${proxyPath}/map_module/mapeditors/maps_service_browser_redirect.json`, {
            params: params as {} // cast ContactsIndexParams into object
        }).pipe(
            map(data => {
                return data;
            }),
            catchError((error: any) => {
                return of({
                    errors: true
                });
            })
        )
    }

}
