import { PaginateOrScroll } from '../../layouts/coreui/paginator/paginator.interface';
import { AcknowledgementTypes } from './acknowledgement-types.enum';

export interface AcknowledgementObject extends AcknowledgedHost {
    commentDataHtml: string
}

export interface AcknowledgedHost {
    hostname: string
    acknowledgement_type: number
    author_name: string
    comment_data: string
    comment_data_html: string
    entry_time: string
    is_sticky: boolean
    notify_contacts: boolean
    persistent_comment: boolean
    state: number
}

export interface AcknowledgedService extends AcknowledgedHost {
    service_description: string
}


export interface HostBrowserAcknowledgement {
    hostname: string
    entry_time: string
    entry_time_usec: number
    state: number
    author_name: string
    comment_data: string
    acknowledgement_type: number
    is_sticky: boolean
    persistent_comment: boolean
    notify_contacts: boolean
    comment_data_html: string
}

export interface ServiceBrowserAcknowledgement {
    hostname: string
    service_description: string
    entry_time: string
    entry_time_usec: number
    state: number
    author_name: string
    comment_data: string
    acknowledgement_type: number
    is_sticky: boolean
    persistent_comment: boolean
    notify_contacts: boolean
    comment_data_html: string
}


/**********************
 *    Host action    *
 **********************/
export interface AcknowledgementsHostParams {
    angular: true,
    scroll: boolean,
    sort: string,
    page: number,
    direction: 'asc' | 'desc' | '', // asc or desc
    'filter[AcknowledgementHosts.hostname]': string,
    'filter[AcknowledgementHosts.comment_data]': string,
    'filter[AcknowledgementHosts.state][]': string[],
    'filter[AcknowledgementHosts.author_name]': string,
    'filter[from]': Date | string,
    'filter[to]': Date | string,
}

export function getDefaultAcknowledgementsHostParams(): AcknowledgementsHostParams {
    let now = new Date();
    return {
        angular: true,
        scroll: true,
        sort: 'AcknowledgementHosts.entry_time',
        page: 1,
        direction: 'desc',
        'filter[AcknowledgementHosts.hostname]': '',
        'filter[AcknowledgementHosts.comment_data]': '',
        'filter[AcknowledgementHosts.state][]': [],
        'filter[AcknowledgementHosts.author_name]': '',
        'filter[from]': new Date(now.getTime() - (3600 * 24 * 1000 * 30)),
        'filter[to]': new Date(now.getTime() + (3600 * 24 * 5)),
    }
}

export interface AcknowledgementsHostRoot extends PaginateOrScroll {
    all_acknowledgements: {
        AcknowledgedHost: AcknowledgedHostList
    }[]
}

export interface AcknowledgedHostList {
    acknowledgement_type: AcknowledgementTypes
    author_name: string
    comment_data: string
    entry_time: string
    is_sticky: boolean
    notify_contacts: boolean
    persistent_comment: boolean
    state: number
}

/**********************
 *    Service action    *
 **********************/
export interface AcknowledgementsServiceParams {
    angular: true,
    scroll: boolean,
    sort: string,
    page: number,
    direction: 'asc' | 'desc' | '', // asc or desc
    'filter[AcknowledgementServices.service_description]': string, // uuid
    'filter[AcknowledgementServices.comment_data]': string,
    'filter[AcknowledgementServices.state][]': string[],
    'filter[AcknowledgementServices.author_name]': string,
    'filter[from]': Date | string,
    'filter[to]': Date | string,
}

export function getDefaultAcknowledgementsServiceParams(): AcknowledgementsServiceParams {
    let now = new Date();
    return {
        angular: true,
        scroll: true,
        sort: 'AcknowledgementServices.entry_time',
        page: 1,
        direction: 'desc',
        'filter[AcknowledgementServices.service_description]': '',
        'filter[AcknowledgementServices.comment_data]': '',
        'filter[AcknowledgementServices.state][]': [],
        'filter[AcknowledgementServices.author_name]': '',
        'filter[from]': new Date(now.getTime() - (3600 * 24 * 1000 * 30)),
        'filter[to]': new Date(now.getTime() + (3600 * 24 * 5)),
    }
}

export interface AcknowledgementsServiceRoot extends PaginateOrScroll {
    all_acknowledgements: {
        AcknowledgedService: AcknowledgedServiceList
    }[]
    hostUuid: string
}

export interface AcknowledgedServiceList {
    acknowledgement_type: AcknowledgementTypes
    author_name: string
    comment_data: string
    entry_time: string
    is_sticky: boolean
    notify_contacts: boolean
    persistent_comment: boolean
    state: number
}
