import { inject, Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { PROXY_PATH } from '../../tokens/proxy-path.token';
import { map, Observable } from 'rxjs';
import {
    AcknowledgementObject,
    AcknowledgementsHostParams,
    AcknowledgementsHostRoot,
    AcknowledgementsServiceParams,
    AcknowledgementsServiceRoot
} from './acknowledgement.interface';

@Injectable({
    providedIn: 'root'
})
export class AcknowledgementsService {
    private readonly http = inject(HttpClient);
    private readonly proxyPath = inject(PROXY_PATH);

    constructor() {
    }

    public getAcknowledgementTooltipDetails(objectUuid: string, type: 'hosts' | 'services'): Observable<AcknowledgementObject> {
        const proxyPath = this.proxyPath;

        const params: any = {
            angular: true,
            disableGlobalLoader: true
        };

        if (type === 'hosts') {
            params['uuid'] = objectUuid;
        } else {
            params['serviceuuid'] = objectUuid;
        }

        return this.http.get<{
            acknowledgement: AcknowledgementObject
        }>(`${proxyPath}/${type}/browser.json`, {
            params: params
        }).pipe(
            map(data => {
                return data.acknowledgement
            })
        )
    }

    /**********************
     *    Host action    *
     **********************/
    public getAcknowledgementsHost(params: AcknowledgementsHostParams): Observable<AcknowledgementsHostRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<AcknowledgementsHostRoot>(`${proxyPath}/acknowledgements/host.json`, {
            params: params as {}
        }).pipe(
            map(data => {
                return data;
            })
        )
    }

    /**********************
     *   Service action   *
     **********************/
    public getAcknowledgementsService(params: AcknowledgementsServiceParams): Observable<AcknowledgementsServiceRoot> {
        const proxyPath = this.proxyPath;
        return this.http.get<AcknowledgementsServiceRoot>(`${proxyPath}/acknowledgements/service.json`, {
            params: params as {}
        }).pipe(
            map(data => {
                return data;
            })
        )
    }

}
