import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, Input } from '@angular/core';
import { IconProp } from '@fortawesome/fontawesome-svg-core';
import { AutomapHoststatus } from '../../../services/services.interface';
import { TranslocoService } from '@jsverse/transloco';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { NgClass } from '@angular/common';

@Component({
    selector: 'oitc-hoststatus-icon-automap',
    imports: [
        FaIconComponent,
        NgClass
    ],
    templateUrl: './hoststatus-icon-automap.component.html',
    styleUrl: './hoststatus-icon-automap.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class HoststatusIconAutomapComponent {
    public icon: IconProp = ['fas', 'square'];
    public iconColor: string = 'text-primary';
    public isHardstate: boolean = true;
    public currentState: number = 2;
    public opacity: string = '';


    private _hoststatus?: AutomapHoststatus;

    private readonly TranslocoService = inject(TranslocoService);
    private cdr = inject(ChangeDetectorRef);

    @Input()
    set hoststatus(value: AutomapHoststatus | undefined) {
        this._hoststatus = value;
        this.onHoststatusChange();
    }

    get hoststatus(): AutomapHoststatus | undefined {
        return this._hoststatus;
    }


    onHoststatusChange() {
        if (this.hoststatus) {
            this.cdr.markForCheck();

            this.icon = ['fas', 'square'];

            this.isHardstate = Boolean(this.hoststatus.is_hardstate);
            this.currentState = Number(this.hoststatus.current_state);
            this.opacity = '';

            this.iconColor = 'unreachable';
            switch (this.hoststatus.current_state) {
                case 0:
                    this.iconColor = 'up';
                    break;
                case 1:
                    this.iconColor = 'down';
                    break;
                default:
                    this.iconColor = 'unreachable';
                    break;
            }

            if (this.isHardstate) {
                this.opacity = '';
            } else {
                this.opacity = 'opacity-50';
            }

            if (this.hoststatus.problem_has_been_acknowledged) {
                this.icon = ['fas', 'user'];
            }

            if (Number(this.hoststatus.scheduled_downtime_depth) > 0) {
                this.icon = ['fas', 'power-off'];
            }

        }
    }
}
