import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, Input } from '@angular/core';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { IconProp } from '@fortawesome/fontawesome-svg-core';
import { TranslocoService } from '@jsverse/transloco';
import { NgClass } from '@angular/common';
import { AutomapServicesstatus } from '../../../services/services.interface';

@Component({
    selector: 'oitc-servicestatus-icon-automap',
    imports: [
        FaIconComponent,
        NgClass
    ],
    templateUrl: './servicestatus-icon-automap.component.html',
    styleUrl: './servicestatus-icon-automap.component.css',
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class ServicestatusIconAutomapComponent {

    public icon: IconProp = ['fas', 'square'];
    public iconColor: string = 'text-primary';
    public isHardstate: boolean = true;
    public currentState: number = 3;
    public opacity: string = '';


    private _servicestatus?: AutomapServicesstatus;

    private readonly TranslocoService = inject(TranslocoService);
    private cdr = inject(ChangeDetectorRef);

    @Input()
    set servicestatus(value: AutomapServicesstatus | undefined) {
        this._servicestatus = value;
        this.onServicestatusChange();
    }

    get servicestatus(): AutomapServicesstatus | undefined {
        return this._servicestatus;
    }


    onServicestatusChange() {
        if (this.servicestatus) {
            this.cdr.markForCheck();

            this.icon = ['fas', 'square'];

            this.isHardstate = Boolean(this.servicestatus.is_hardstate);
            this.currentState = Number(this.servicestatus.current_state);
            this.opacity = '';

            this.iconColor = 'unknown';
            switch (this.servicestatus.current_state) {
                case 0:
                    this.iconColor = 'ok';
                    break;
                case 1:
                    this.iconColor = 'warning';
                    break;
                case 2:
                    this.iconColor = 'critical';
                    break;
                default:
                    this.iconColor = 'unknown';
                    break;
            }

            if (this.isHardstate) {
                this.opacity = '';
            } else {
                this.opacity = 'opacity-50';
            }

            if (this.servicestatus.problem_has_been_acknowledged) {
                this.icon = ['fas', 'user'];
            }

            if (Number(this.servicestatus.scheduled_downtime_depth) > 0) {
                this.icon = ['fas', 'power-off'];
            }

        }
    }

}
