import { ChangeDetectionStrategy, ChangeDetectorRef, Component, inject, OnDestroy, OnInit } from '@angular/core';
import {
    AlertComponent,
    BadgeComponent,
    BorderDirective,
    ButtonGroupComponent,
    ButtonToolbarComponent,
    CardBodyComponent,
    CardComponent,
    CardHeaderComponent,
    CardTitleDirective,
    ColComponent,
    ModalService,
    NavComponent,
    NavItemComponent,
    RowComponent,
    TableDirective,
    TooltipDirective
} from '@coreui/angular';

import { TranslocoDirective, TranslocoPipe, TranslocoService } from '@jsverse/transloco';
import { BehaviorSubject, Subscription } from 'rxjs';
import { NotyService } from '../../../layouts/coreui/noty.service';
import { ActivatedRoute, Router, RouterLink } from '@angular/router';
import { HostsService } from '../../hosts/hosts.service';
import { ServicesService } from '../services.service';
import { HistoryService } from '../../../history.service';
import {
    ServiceBrowserMenuConfig,
    ServicesBrowserMenuComponent
} from '../services-browser-menu/services-browser-menu.component';
import { TimezoneConfiguration as TimezoneObject, TimezoneService } from '../../../services/timezone.service';

import { SelectKeyValueString } from '../../../layouts/primeng/select.interface';
import { ServiceBrowserTabs } from '../services.enum';
import { PermissionsService } from '../../../permissions/permissions.service';
import { ExternalCommandsEnum } from '../../../enums/external-commands.enum';
import {
    ExternalCommandsService,
    ServiceProcessCheckResultItem,
    ServiceResetItem
} from '../../../services/external-commands.service';
import { CancelAllItem } from '../../downtimes/cancel-hostdowntime-modal/cancel-hostdowntime.interface';
import { HostBrowserDowntime, ServiceBrowserDowntime } from '../../downtimes/downtimes.interface';
import { DELETE_SERVICE_TOKEN } from '../../../tokens/delete-injection.token';
import { DELETE_ACKNOWLEDGEMENT_SERVICE_TOKEN } from '../../../tokens/delete-acknowledgement-injection.token';
import { DowntimesService } from '../../downtimes/downtimes.service';
import { AcknowledgementsService } from '../../acknowledgements/acknowledgements.service';

import { TitleService } from '../../../services/title.service';
import { BrowserLoaderComponent } from '../../../layouts/primeng/loading/browser-loader/browser-loader.component';
import { FormsModule } from '@angular/forms';
import { ServicesBrowserRoot, ServicestatusBrowser } from '../services.interface';
import { GenericUnixtimerange } from '../../../generic.interfaces';
import { AsyncPipe, NgClass, NgIf } from '@angular/common';
import { ObjectUuidComponent } from '../../../layouts/coreui/object-uuid/object-uuid.component';
import { XsButtonDirective } from '../../../layouts/coreui/xsbutton-directive/xsbutton.directive';
import { FaIconComponent } from '@fortawesome/angular-fontawesome';
import { BackButtonDirective } from '../../../directives/back-button.directive';
import { PermissionDirective } from '../../../permissions/permission.directive';
import { ServiceStatusNamePipe } from '../../../pipes/service-status-name.pipe';
import { TrustAsHtmlPipe } from '../../../pipes/trust-as-html.pipe';
import { HostsBrowserRoot } from '../../hosts/hosts.interface';
import { HoststatusSimpleIconComponent } from '../../hosts/hoststatus-simple-icon/hoststatus-simple-icon.component';
import {
    ServiceResetChecktimeModalComponent
} from '../../../components/services/service-reset-checktime-modal/service-reset-checktime-modal.component';
import {
    ServiceMaintenanceModalComponent
} from '../../../components/services/service-maintenance-modal/service-maintenance-modal.component';
import {
    ServiceAcknowledgeModalComponent
} from '../../../components/services/service-acknowledge-modal/service-acknowledge-modal.component';
import {
    ServicesProcessCheckresultModalComponent
} from '../../../components/services/services-process-checkresult-modal/services-process-checkresult-modal.component';
import {
    HostsEnableFlapdetectionModalComponent
} from '../../../components/hosts/hosts-enable-flapdetection-modal/hosts-enable-flapdetection-modal.component';
import {
    HostsDisableFlapdetectionModalComponent
} from '../../../components/hosts/hosts-disable-flapdetection-modal/hosts-disable-flapdetection-modal.component';
import {
    CancelHostdowntimeModalComponent
} from '../../downtimes/cancel-hostdowntime-modal/cancel-hostdowntime-modal.component';
import {
    CancelServicedowntimeModalComponent
} from '../../downtimes/cancel-servicedowntime-modal/cancel-servicedowntime-modal.component';
import { ServicesBrowserChartComponent } from './services-browser-chart/services-browser-chart.component';

@Component({
    selector: 'oitc-services-browser',
    imports: [
        ServicesBrowserMenuComponent,
        FormsModule,
        BrowserLoaderComponent,
        CardComponent,
        CardHeaderComponent,
        CardTitleDirective,
        NgIf,
        TranslocoDirective,
        NavComponent,
        ObjectUuidComponent,
        NavItemComponent,
        XsButtonDirective,
        FaIconComponent,
        BackButtonDirective,
        PermissionDirective,
        NgClass,
        CardBodyComponent,
        RowComponent,
        ColComponent,
        ServiceStatusNamePipe,
        ButtonToolbarComponent,
        ButtonGroupComponent,
        TooltipDirective,
        AsyncPipe,
        TranslocoPipe,
        BorderDirective,
        TrustAsHtmlPipe,
        AlertComponent,
        RouterLink,
        TableDirective,
        BadgeComponent,
        HoststatusSimpleIconComponent,
        ServiceResetChecktimeModalComponent,
        ServiceMaintenanceModalComponent,
        ServiceAcknowledgeModalComponent,
        ServicesProcessCheckresultModalComponent,
        HostsEnableFlapdetectionModalComponent,
        HostsDisableFlapdetectionModalComponent,
        CancelHostdowntimeModalComponent,
        CancelServicedowntimeModalComponent,
        ServicesBrowserChartComponent
    ],
    templateUrl: './services-browser.component.html',
    styleUrl: './services-browser.component.css',
    providers: [
        {provide: DELETE_SERVICE_TOKEN, useClass: DowntimesService}, // Inject the DowntimesService into the CancelAllModalComponent
        {provide: DELETE_ACKNOWLEDGEMENT_SERVICE_TOKEN, useClass: AcknowledgementsService} // Inject the DowntimesService into the DeleteAllModalComponent
    ],
    changeDetection: ChangeDetectionStrategy.OnPush
})
export class ServicesBrowserComponent implements OnInit, OnDestroy {

    public uuid: string = '';

    public serviceBrowserConfig?: ServiceBrowserMenuConfig;
    public timezone!: TimezoneObject;

    public result?: ServicesBrowserRoot;
    public hoststatus?: HostsBrowserRoot;
    public lastUpdated: Date = new Date(); // Used to tell child components to reload data
    public availableDataSources: SelectKeyValueString[] = []; // The API result is not as good

    public selectedTab: ServiceBrowserTabs = ServiceBrowserTabs.StatusInformation;
    public selectedItems: any[] = [];

    private readonly timerange$$ = new BehaviorSubject<GenericUnixtimerange>({start: 0, end: 0});
    public readonly timerange$ = this.timerange$$.asObservable();
    private graphTimerange: GenericUnixtimerange = {start: 0, end: 0};

    public cssClass: string = 'bg-unknown';
    public textClass: string = 'unknown';

    private subscriptions: Subscription = new Subscription();
    private readonly HostsService = inject(HostsService);
    private readonly ServicesService = inject(ServicesService);
    private readonly notyService = inject(NotyService);
    private readonly TimezoneService = inject(TimezoneService);
    public readonly PermissionsService = inject(PermissionsService);
    private router = inject(Router);
    private route = inject(ActivatedRoute);
    private readonly HistoryService: HistoryService = inject(HistoryService);
    private readonly ExternalCommandsService = inject(ExternalCommandsService);
    private readonly TranslocoService = inject(TranslocoService);
    private readonly TitleService: TitleService = inject(TitleService);
    private readonly modalService = inject(ModalService);
    private readonly DowntimesService = inject(DowntimesService);
    private readonly AcknowledgementsService = inject(AcknowledgementsService);
    private cdr = inject(ChangeDetectorRef);

    constructor() {
    }

    public ngOnInit(): void {
        this.getUserTimezone();

        this.uuid = String(this.route.snapshot.paramMap.get('uuid'));
        this.loadService();
    }

    public ngOnDestroy(): void {
        this.subscriptions.unsubscribe();
    }

    private getUserTimezone() {
        this.subscriptions.add(this.TimezoneService.getTimezoneConfiguration().subscribe(data => {
            this.timezone = data;
            this.cdr.markForCheck();
        }));
    }

    public loadService() {
        this.cdr.markForCheck();
        this.subscriptions.add(this.ServicesService.getServiceBrowser(this.uuid).subscribe((result) => {
            this.cdr.markForCheck();

            const hostUuid = result.servicestatus.hostname;

            // Load host status
            this.subscriptions.add(this.HostsService.getHostBrowser(hostUuid).subscribe((result) => {
                this.cdr.markForCheck();
                this.hoststatus = result;
            }));

            // Define the configuration for the ServiceBrowserMenuComponent because we know the hostUuid now
            this.serviceBrowserConfig = {
                hostUuid: result.servicestatus.hostname,
                serviceUuid: this.uuid,
                showReschedulingButton: true,
                rescheduleCallback: () => {
                    console.log('implement callback')
                },
                showBackButton: false
            };

            this.result = result;

            switch (result.servicestatus.current_state) {
                case 0:
                    this.cssClass = 'bg-ok';
                    this.textClass = 'ok';
                    break;
                case 1:
                    this.cssClass = 'bg-warning';
                    this.textClass = 'warning';
                    break;
                case 2:
                    this.cssClass = 'bg-critical';
                    this.textClass = 'critical';
                    break;

                default:
                    this.cssClass = 'bg-unknown';
                    this.textClass = 'unknown';
                    break;
            }

            for (let key in result.Perfdata) {
                const gauge = result.Perfdata[key];
                this.availableDataSources.push({
                    key: key, // load this datasource
                    value: key // Name of the metric to display in select
                });
            }

            this.lastUpdated = new Date();

            // Update the title.
            let newTitle: string = this.result.servicestatus.host_mapping.display_name + '/' + this.result.servicestatus.service_mapping.display_name;
            this.TitleService.setTitle(`${newTitle} | ` + this.TranslocoService.translate('Service Browser'));
        }));
    }

    public changeTab(newTab: ServiceBrowserTabs): void {
        this.selectedTab = newTab;
    }

    public onMassActionComplete(success: boolean) {
        if (success) {
            this.loadService();
        }
    }

    public resetChecktime(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        const commands: ServiceResetItem[] = [
            {
                command: ExternalCommandsEnum.rescheduleService,
                hostUuid: servicestatus.hostname,
                serviceUuid: servicestatus.service_description,
            }
        ];

        if (commands.length === 0) {
            const message = this.TranslocoService.translate('No items selected!');
            this.notyService.genericError(message);
            return;
        }

        commands.forEach(command => {
            this.subscriptions.add(this.ExternalCommandsService.reset_check_time(command).subscribe((result) => {
            }));
        });

        const title = this.TranslocoService.translate('Reschedule');
        const msg: string = this.TranslocoService.translate('Commands added successfully to queue');

        this.notyService.genericSuccess(msg, title);

        setTimeout(() => {
            this.onMassActionComplete(true);
        }, 5000);
    }

    public toggleDowntimeModal(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        if (this.result) {
            this.selectedItems.push({
                command: ExternalCommandsEnum.submitServiceDowntime,
                hostUuid: servicestatus.hostname,
                serviceUuid: servicestatus.service_description,
                start: 0,
                end: 0,
                author: this.result.username,
                comment: '',
            });

            this.modalService.toggle({
                show: true,
                id: 'serviceMaintenanceModal',
            });
        }
    }

    public acknowledgeStatus(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        if (this.result) {
            this.selectedItems.push({
                command: ExternalCommandsEnum.submitServicestateAck,
                hostUuid: servicestatus.hostname,
                serviceUuid: servicestatus.service_description,
                sticky: 0,
                notify: false,
                author: this.result.username,
                comment: '',
            });

            this.modalService.toggle({
                show: true,
                id: 'serviceAcknowledgeModal',
            });
        }

        setTimeout(() => {
            this.loadService()
        }, 5000);
    }

    public processCheckResult(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        const item: ServiceProcessCheckResultItem = {
            command: ExternalCommandsEnum.commitPassiveServiceResult,
            maxCheckAttempts: servicestatus.max_check_attempts,
            hostUuid: servicestatus.hostname,
            serviceUuid: servicestatus.service_description,
            status_code: 0, // Will be overwritten with the modals default value
            plugin_output: '', // Will be overwritten with the modals default value
            forceHardstate: false, // Will be overwritten with the modals default value
            long_output: ''
        };

        this.selectedItems.push(item);

        this.modalService.toggle({
            show: true,
            id: 'serviceProcessCheckresultModal',
        });
    }

    public enableFlapdetection(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        this.selectedItems.push({
            command: ExternalCommandsEnum.enableOrDisableServiceFlapdetection,
            hostUuid: servicestatus.hostname,
            serviceUuid: servicestatus.service_description,
            condition: 1
        });

        this.modalService.toggle({
            show: true,
            id: 'hostEnableFlapdetectionModal',
        });
    }

    public disableFlapdetection(servicestatus: ServicestatusBrowser) {
        this.selectedItems = [];

        this.selectedItems.push({
            command: ExternalCommandsEnum.enableOrDisableServiceFlapdetection,
            hostUuid: servicestatus.hostname,
            serviceUuid: servicestatus.service_description,
            condition: 0
        });

        this.modalService.toggle({
            show: true,
            id: 'hostDisableFlapdetectionModal',
        });
    }

    public toggleCancelHostDowntimeModal(hostDowntime: HostBrowserDowntime) {
        this.selectedItems = [];

        const item: CancelAllItem[] = [{
            id: hostDowntime.internal_downtime_id
        }];

        // Pass selection to the modal
        this.selectedItems = item;

        // open modal
        this.modalService.toggle({
            show: true,
            id: 'cancelHostAllModal',
        });
    }

    public toggleCancelServiceDowntimeModal(serviceDowntime: ServiceBrowserDowntime) {
        this.selectedItems = [];

        const item: CancelAllItem[] = [{
            id: serviceDowntime.internal_downtime_id
        }];

        // Pass selection to the modal
        this.selectedItems = item;

        // open modal
        this.modalService.toggle({
            show: true,
            id: 'cancelServiceAllModal',
        });
    }

    public onToggleRescheduling(event: boolean) {
        // Service browser menu wants to toggle a rescheduling
        if (this.result) {
            this.resetChecktime(this.result.servicestatus);
        }
    }

    protected readonly String = String;
    protected readonly Number = Number;
    protected readonly ServiceBrowserTabs = ServiceBrowserTabs;
    protected readonly Boolean = Boolean;
}
